"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.register = void 0;
const PConst = require("../protocol.const");
const node_1 = require("vscode-languageserver/node");
const modifiers_1 = require("../utils/modifiers");
const shared_1 = require("@volar/shared");
const getSymbolKind = (kind) => {
    switch (kind) {
        case PConst.Kind.module: return node_1.SymbolKind.Module;
        case PConst.Kind.class: return node_1.SymbolKind.Class;
        case PConst.Kind.enum: return node_1.SymbolKind.Enum;
        case PConst.Kind.interface: return node_1.SymbolKind.Interface;
        case PConst.Kind.method: return node_1.SymbolKind.Method;
        case PConst.Kind.memberVariable: return node_1.SymbolKind.Property;
        case PConst.Kind.memberGetAccessor: return node_1.SymbolKind.Property;
        case PConst.Kind.memberSetAccessor: return node_1.SymbolKind.Property;
        case PConst.Kind.variable: return node_1.SymbolKind.Variable;
        case PConst.Kind.const: return node_1.SymbolKind.Variable;
        case PConst.Kind.localVariable: return node_1.SymbolKind.Variable;
        case PConst.Kind.function: return node_1.SymbolKind.Function;
        case PConst.Kind.localFunction: return node_1.SymbolKind.Function;
        case PConst.Kind.constructSignature: return node_1.SymbolKind.Constructor;
        case PConst.Kind.constructorImplementation: return node_1.SymbolKind.Constructor;
    }
    return node_1.SymbolKind.Variable;
};
function register(languageService, getTextDocument) {
    return (uri) => {
        const document = getTextDocument(uri);
        if (!document)
            return [];
        const fileName = shared_1.uriToFsPath(document.uri);
        const barItems = languageService.getNavigationTree(fileName);
        const result = [];
        convertNavTree(document, barItems);
        return result;
        function convertNavTree(document, item) {
            var _a;
            let shouldInclude = shouldInclueEntry(item);
            if (!shouldInclude && !((_a = item.childItems) === null || _a === void 0 ? void 0 : _a.length)) {
                return false;
            }
            for (const span of item.spans) {
                const range = node_1.Range.create(document.positionAt(span.start), document.positionAt(span.start + span.length));
                const selectionRange = item.nameSpan ? node_1.Range.create(document.positionAt(item.nameSpan.start), document.positionAt(item.nameSpan.start + item.nameSpan.length)) : range;
                const symbolInfo = node_1.DocumentSymbol.create(item.text, '', getSymbolKind(item.kind), range, selectionRange);
                const kindModifiers = modifiers_1.parseKindModifier(item.kindModifiers);
                if (kindModifiers.has(PConst.KindModifiers.depreacted)) {
                    symbolInfo.deprecated = true;
                }
                if (shouldInclude) {
                    result.push(symbolInfo);
                }
            }
            return shouldInclude;
        }
        function shouldInclueEntry(item) {
            if (item.kind === PConst.Kind.alias) {
                return false;
            }
            return !!(item.text && item.text !== '<function>' && item.text !== '<class>');
        }
    };
}
exports.register = register;
//# sourceMappingURL=documentSymbol.js.map