"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.boundSpanToLocationLinks = exports.entriesToLocationLinks = exports.entriesToLocations = void 0;
const node_1 = require("vscode-languageserver/node");
const shared_1 = require("@volar/shared");
function entriesToLocations(entries, getTextDocument) {
    const locations = [];
    for (const entry of entries) {
        const entryUri = shared_1.fsPathToUri(entry.fileName);
        const doc = getTextDocument(entryUri);
        if (!doc)
            continue;
        const range = node_1.Range.create(doc.positionAt(entry.textSpan.start), doc.positionAt(entry.textSpan.start + entry.textSpan.length));
        const uri = shared_1.fsPathToUri(entry.fileName);
        const location = node_1.Location.create(uri, range);
        locations.push(location);
    }
    return locations;
}
exports.entriesToLocations = entriesToLocations;
function entriesToLocationLinks(entries, getTextDocument) {
    const locations = [];
    for (const entry of entries) {
        const entryUri = shared_1.fsPathToUri(entry.fileName);
        const doc = getTextDocument(entryUri);
        if (!doc)
            continue;
        const targetSelectionRange = node_1.Range.create(doc.positionAt(entry.textSpan.start), doc.positionAt(entry.textSpan.start + entry.textSpan.length));
        const targetRange = entry.contextSpan ? node_1.Range.create(doc.positionAt(entry.contextSpan.start), doc.positionAt(entry.contextSpan.start + entry.contextSpan.length)) : targetSelectionRange;
        const originSelectionRange = entry.originalTextSpan ? node_1.Range.create(doc.positionAt(entry.originalTextSpan.start), doc.positionAt(entry.originalTextSpan.start + entry.originalTextSpan.length)) : undefined;
        const uri = shared_1.fsPathToUri(entry.fileName);
        const location = node_1.LocationLink.create(uri, targetRange, targetSelectionRange, originSelectionRange);
        locations.push(location);
    }
    return locations;
}
exports.entriesToLocationLinks = entriesToLocationLinks;
function boundSpanToLocationLinks(info, originalDoc, getTextDocument) {
    const locations = [];
    if (!info.definitions)
        return locations;
    const originSelectionRange = node_1.Range.create(originalDoc.positionAt(info.textSpan.start), originalDoc.positionAt(info.textSpan.start + info.textSpan.length));
    for (const entry of info.definitions) {
        const entryUri = shared_1.fsPathToUri(entry.fileName);
        const doc = getTextDocument(entryUri);
        if (!doc)
            continue;
        const targetSelectionRange = node_1.Range.create(doc.positionAt(entry.textSpan.start), doc.positionAt(entry.textSpan.start + entry.textSpan.length));
        const targetRange = entry.contextSpan ? node_1.Range.create(doc.positionAt(entry.contextSpan.start), doc.positionAt(entry.contextSpan.start + entry.contextSpan.length)) : targetSelectionRange;
        const uri = shared_1.fsPathToUri(entry.fileName);
        const location = node_1.LocationLink.create(uri, targetRange, targetSelectionRange, originSelectionRange);
        locations.push(location);
    }
    return locations;
}
exports.boundSpanToLocationLinks = boundSpanToLocationLinks;
//# sourceMappingURL=transforms.js.map